#!/usr/bin/perl

eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use warnings;

use lib 'lib';
use File::DXF;
use File::DXF::Util qw /dxf read_DXF/;

# Takes DXF 3DFACE entities and joins into a POLYLINE polyface mesh

die "usage: $0 INPUT.DXF OUTPUT.DXF" unless @ARGV == 2;

my $dxf_in = new File::DXF;
my $text = read_DXF ($ARGV[0], 'CP1252');
   $dxf_in->Process ($text);

my @report = $dxf_in->{ENTITIES}->Report ('3DFACE');

print STDERR '3DFACE entities: '. scalar (@report) ."\n";

# get list of nodes used
my $used_nodes;
for my $_3dface (@report)
{
    # Packed() returns a stringified version of a 3dcoordinate
    $used_nodes->{$_3dface->Packed (0)} = $_3dface->Coordinates (0);
    $used_nodes->{$_3dface->Packed (1)} = $_3dface->Coordinates (1);
    $used_nodes->{$_3dface->Packed (2)} = $_3dface->Coordinates (2);
    $used_nodes->{$_3dface->Packed (3)} = $_3dface->Coordinates (3);
}

# build a lookup table of nodes
# DXF nodes are implicitly numbered, starting at 1
my $lookup;
my $lookup_index = 1;
for my $packed (sort keys %{$used_nodes})
{
    $lookup->{$packed} = $lookup_index;
    $lookup_index++;
}

# write the DXF file

my $dxf_out = new File::DXF;
$dxf_out->{HEADER} = new File::DXF::HEADER;
$dxf_out->{ENTITIES} = new File::DXF::ENTITIES;
$dxf_out->{BLOCKS} = new File::DXF::BLOCKS;

my $polyline = {type => 'POLYLINE', _data => [
    [8, 'mesh'],
    [62, 0],
    [66, 1],
    [10, 0.0],
    [20, 0.0],
    [30, 0.0],
    [70, 64],
    [71, scalar (keys %{$lookup})],
    [72, scalar (@report)]
]};

push @{$dxf_out->{ENTITIES}}, $polyline;

# dump nodes
for my $packed (sort keys %{$lookup})
{
    my $node = $used_nodes->{$packed};

    my $vertex = {type => 'VERTEX', _data => [
        [8, 'mesh'],
        [10, $node->[0]],
        [20, $node->[1]],
        [30, $node->[2]],
        [70, 192]
    ]};

    push @{$dxf_out->{ENTITIES}}, $vertex;
}

# dump triangles
for my $_3dface (@report)
{
    # get four node ids
    my $nodes = [map {$_3dface->Packed ($_)} (0 .. 3)];

    # transform from a quad to a triangle if consecutive nodes coincide
    my $nodes_new = [];
    for my $node_id (0 .. 3)
    {
        next if $nodes->[$node_id] eq $nodes->[$node_id -1];
        push @{$nodes_new}, $nodes->[$node_id];
    }
    $nodes = $nodes_new;

    next unless scalar @{$nodes} > 2;
    my $vertex = {type => 'VERTEX', _data => [
        [8, 'mesh'],
        [62, 0],
        [10, 0.0],
        [20, 0.0],
        [30, 0.0],
        [62, $_3dface->Colour],
        [70, 128],
        [71, $lookup->{$nodes->[0]}],
        [72, $lookup->{$nodes->[1]}],
        [73, $lookup->{$nodes->[2]}],
    ]};
    push @{$vertex->{_data}}, [74, $lookup->{$nodes->[3]}] if defined $nodes->[3];

    push @{$dxf_out->{ENTITIES}}, $vertex;
}

push @{$dxf_out->{ENTITIES}}, {type => 'SEQEND', _data => []};

# write out original block table as-is
for my $key (keys %{$dxf_in->{BLOCKS}})
{
    $dxf_out->{BLOCKS}->{$key} = $dxf_in->{BLOCKS}->{$key};
}

# write out original entities as-is except 3dfaces
for my $entity (@{$dxf_in->{ENTITIES}})
{
    push @{$dxf_out->{ENTITIES}}, $entity unless $entity->{type} eq '3DFACE';
}

open DXF, '>'. $ARGV[1];
binmode DXF, ":crlf";
print DXF $dxf_out->DXF;
close DXF;

0;
