#!/usr/bin/perl

eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use warnings;

use lib 'lib';
use File::DXF;
use File::DXF::Util qw /dxf read_DXF/;
use File::DXF::Math qw /add_3d rotate_3d PI/;
use Storable qw/ dclone /;

die "usage: $0 INPUT.DXF OUTPUT.DXF" unless @ARGV == 2;

my $dxf_in = new File::DXF;
my $text = read_DXF ($ARGV[0], 'CP1252');
   $dxf_in->Process ($text);

my @report = $dxf_in->{ENTITIES}->Report ('INSERT');

print STDERR 'INSERT entities: '. scalar (@report) ."\n";

# write the DXF file

my $dxf_out = new File::DXF;
$dxf_out->{HEADER} = new File::DXF::HEADER;
$dxf_out->{ENTITIES} = new File::DXF::ENTITIES;

my $data_out;

for my $insert (@report)
{
    my $shift = [$insert->{x}, $insert->{y}, $insert->{z}];
    my $rotation = PI() * $insert->{rotation} / 180;
    my $data = dclone $dxf_in->{BLOCKS}->{$insert->{name}};

    my ($x, $y, $z);

    while (@{$data})
    {
        my ($code, $string) = File::DXF::_next ($data);

        unless ($code =~ /^[123][0-9]$/)
        {
            push @{$data_out}, [$code, $string];
            next;
        }

        $x = $string if ($code =~ /^1[0-9]$/);
        $y = $string if ($code =~ /^2[0-9]$/);
        $z = $string if ($code =~ /^3[0-9]$/);

        # a Z code, capture the suffix
        if ($code =~ /^3([0-9])$/)
        {
            my $point = rotate_3d ($rotation, [$x, $y, $z]);
            $point = add_3d ($point, $shift);

            push @{$data_out}, ["1$1", $point->[0]], ["2$1", $point->[1]], ["3$1", $point->[2]];
        }
    }
    $dxf_out->{ENTITIES}->Process ($data_out);
}

# write out original entities as-is except inserts
for my $entity (@{$dxf_in->{ENTITIES}})
{
    push @{$dxf_out->{ENTITIES}}, $entity unless $entity->{type} eq 'INSERT';
}

open DXF, '>'. $ARGV[1];
binmode DXF, ":crlf";
print DXF $dxf_out->DXF;
close DXF;

0;
