#!/usr/bin/perl

eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use warnings;

use lib ('lib');
use File::DXF::Util qw /read_DXF/;
use File::DXF;
use File::DXF::ENTITIES;

# Reads a DXF and generates collada geometry.
# only handles POLYLINE 'polyface meshes' and BLOCKS of the same
# BLOCK scale and out-of XY-plane rotation not supported

die "usage: $0 input.dxf output.dae" unless @ARGV == 2;

my $data = read_DXF ($ARGV[0], 'CP1252');

my $dxf = new File::DXF;
   $dxf->Process ($data);

my $xml_instances;
my $xml_geometries;

my $id = 0;

for my $mesh ($dxf->{ENTITIES}->Extract_Meshes)
{
    $xml_geometries .= _collada_mesh ("Mesh$id", $mesh->[0], $mesh->[1]);
    $xml_instances .= _collada_instance ("Mesh$id", [0.0,0.0,0.0], 0.0);
    $id++;
}

my $valid_blocks = {};

for my $key (keys %{$dxf->{BLOCKS}})
{
    my $entities = new File::DXF::ENTITIES;
    $entities->Process ($dxf->{BLOCKS}->{$key});
    my @meshes = $entities->Extract_Meshes;
    my $mesh = shift @meshes;
    next unless defined $mesh->[0];
    $xml_geometries .= _collada_mesh ($key, $mesh->[0], $mesh->[1]);
    $valid_blocks->{$key} = 1;
}

for my $insert ($dxf->{ENTITIES}->Report ('INSERT'))
{
    next unless defined $valid_blocks->{$insert->{name}};
    $xml_instances .= _collada_instance ($insert->{name},
                          [$insert->{x},$insert->{y},$insert->{z}], $insert->{rotation});
}

open DAE, '>'. $ARGV[1];
binmode DAE, ":crlf";

print DAE '<?xml version="1.0" encoding="utf-8"?>'."\n";
print DAE '<COLLADA version="1.4.0" xmlns="http://www.collada.org/2005/11/COLLADASchema">'."\n";

print DAE '<asset>'."\n";
print DAE ' <contributor>'."\n";
print DAE '  <author></author>'."\n";
print DAE '  <authoring_tool>File::DXF</authoring_tool>'."\n";
print DAE '  <comments></comments>'."\n";
print DAE '  <copyright></copyright>'."\n";
print DAE '  <source_data></source_data>'."\n";
print DAE ' </contributor>'."\n";
print DAE ' <created>2012-01-03T16:37:29.985000</created>'."\n";
print DAE ' <modified>2012-01-03T16:37:29.985000</modified>'."\n";
print DAE ' <unit meter="1.00" name="meter"/>'."\n";
print DAE ' <up_axis>Z_UP</up_axis>'."\n";
print DAE '</asset>'."\n";

print DAE '<library_geometries>'."\n";

print DAE $xml_geometries."\n";

print DAE '</library_geometries>'."\n";

print DAE '<library_visual_scenes>'."\n";
print DAE ' <visual_scene id="Scene" name="Scene">'."\n";

print DAE $xml_instances."\n";

print DAE ' </visual_scene>'."\n";
print DAE '</library_visual_scenes>'."\n";
print DAE '<scene>'."\n";
print DAE ' <instance_visual_scene url="#Scene"/>'."\n";
print DAE '</scene>'."\n";
print DAE '</COLLADA>'."\n";

close DAE;

exit 0;

sub _collada_instance
{
    my ($id, $coor, $rotation) = @_;
    $rotation = 0.0 unless $rotation;
    my $xml = '
  <node layer="L1" id="'.$id.'" name="'.$id.'">
   <translate sid="translate">'. join (' ', @{$coor}) .'</translate>
   <rotate sid="rotateZ">0 0 1 '.$rotation.'</rotate>
   <rotate sid="rotateY">0 1 0 0.00000</rotate>
   <rotate sid="rotateX">1 0 0 0.00000</rotate>
   <scale sid="scale">1.00000 1.00000 1.00000</scale>
   <instance_geometry url="#'.$id.'-Geometry"/>
  </node>';
    return $xml;
}

sub _collada_mesh
{
    my ($id, $nodes, $triangles) = @_;
    print STDERR "Writing mesh. ID, nodes, triangles: $id, ".
        scalar @{$nodes}. ', ' .scalar @{$triangles}."\n";
    my $xml = '
 <geometry id="'.$id.'-Geometry" name="'.$id.'-Geometry">
  <mesh>
   <source id="'.$id.'-Geometry-Position">
    <float_array count="'. (scalar (@{$nodes}) * 3) .'" id="'.$id.'-Geometry-Position-array">
'.
    join (' ', @{$nodes}) .'
    </float_array>
    <technique_common>
     <accessor count="'. scalar (@{$nodes}) .'" source="#'.$id.'-Geometry-Position-array" stride="3">
      <param type="float" name="X"></param>
      <param type="float" name="Y"></param>
      <param type="float" name="Z"></param>
     </accessor>
    </technique_common>
   </source>
   <vertices id="'.$id.'-Geometry-Vertex">
    <input semantic="POSITION" source="#'.$id.'-Geometry-Position"/>
   </vertices>
   <polygons count="'. scalar (@{$triangles}) .'">
    <input offset="0" semantic="VERTEX" source="#'.$id.'-Geometry-Vertex"/><p>'.
    join ('</p><p>', @{$triangles}).'
   </p></polygons>
  </mesh>
 </geometry>';
    return $xml;
}

