#!/usr/bin/perl

eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use warnings;

use lib ('lib');
use File::DXF::Util qw /read_DXF/;
use File::DXF::Math qw /scale_3d add_3d area_3d cog_3d green_theorem average_3d/;
use File::DXF;
use 5.010;

# Reads a DXF POLYLINE 'triangle mesh' and calculates the surface area, volume,
# CoG of surface and CoG of volume.  Volume calculations are wrong unless mesh
# is closed manifold with unified normals.

die "usage: $0 input.dxf" unless @ARGV == 1;

my $data = read_DXF ($ARGV[0], 'CP1252');

my $dxf = new File::DXF;
   $dxf->Process ($data);
my $entities = $dxf->{ENTITIES};

my @report = $entities->Report (('POLYLINE', 'VERTEX', 'SEQEND'));

my $state = 'INIT';
my $area_mesh = 0.0;
my $volume_mesh = 0.0;
my $id_triangle;
my $points = [];
my $cog_area = [];
my $cog_volume = [];

my $area_total = 0.0;
my $volume_total = 0.0;
my $cog_area_total = [];
my $cog_volume_total = [];

for my $item (@report)
{
    if ($item->Type eq 'POLYLINE' and $item->Triangular)
    {
        say STDERR "found mesh, ". $item->Nodes ." nodes, ". $item->Triangles ." triangles";
        $state = 'POINTS';
        $area_mesh = 0.0;
        $volume_mesh = 0.0;
        $cog_area = [0.0, 0.0, 0.0];
        $cog_volume = [0.0, 0.0, 0.0];
        $id_triangle = 0;
        $points = ['This space intentionally left blank'];
    }

    if ($state eq 'POINTS' and $item->Type eq 'VERTEX' and $item->Point)
    {
        push @{$points}, [$item->{x}, $item->{y}, $item->{z}];
    }

    if ($state eq 'POINTS' and $item->Type eq 'VERTEX' and $item->Triangle)
    {

	$item->{a} = 0 - $item->{a} if ($item->{a} < 0);
	$item->{b} = 0 - $item->{b} if ($item->{b} < 0);
	$item->{c} = 0 - $item->{c} if ($item->{c} < 0);

        my $a = $points->[$item->{a}];
        my $b = $points->[$item->{b}];
        my $c = $points->[$item->{c}];
        my $centre = average_3d ($a, $b, $c);

        my $area = area_3d ($a, $b, $c);
        $area_mesh += $area;

        my $volume = $centre->[2] * green_theorem ($a, $b, $c);
        $volume_mesh += $volume;

        $cog_area = add_3d ($cog_area, scale_3d ($area, $centre));

        $cog_volume = add_3d ($cog_volume, scale_3d ($volume, cog_3d ($a, $b, $c)));

        # some triangles have four sides
        if (defined $item->{d})
        {
	    $item->{d} = 0 - $item->{d} if ($item->{d} < 0);

            my $d = $points->[$item->{d}];
            my $centre = average_3d ($a, $c, $d);

            my $area = area_3d ($a, $c, $d);
            $area_mesh += $area;

            my $volume = $centre->[2] * green_theorem ($a, $c, $d);
            $volume_mesh += $volume;

            $cog_area = add_3d ($cog_area, scale_3d ($area, $centre));

            $cog_volume = add_3d ($cog_volume, scale_3d ($volume, cog_3d ($a, $c, $d)));
        }
        $id_triangle++;
        print STDERR "\r$id_triangle";
    }

    if ($item->Type eq 'SEQEND')
    {
        $cog_area_total = add_3d ($cog_area_total, $cog_area);

        print STDERR "\r";
        say STDOUT "area: $area_mesh";
        $cog_area = scale_3d ($cog_area, 1/$area_mesh);
        say STDOUT 'CoG of surface: ' . join (', ', @{$cog_area});

        $cog_volume_total = add_3d ($cog_volume_total, $cog_volume);

        say STDOUT "volume: $volume_mesh";
        $volume_mesh = 1 unless $volume_mesh;
        $cog_volume = scale_3d ($cog_volume, 1/$volume_mesh);
        say STDOUT 'CoG of volume: ' . join (', ', @{$cog_volume});
        say STDOUT '';

        $area_total += $area_mesh;
        $volume_total += $volume_mesh;

        $state = 'NEXT';
    }
}

say STDOUT "Total area: $area_total";
$cog_area_total = scale_3d ($cog_area_total, 1/$area_total);
say STDOUT 'CoG of Total surface: ' . join (', ', @{$cog_area_total});

say STDOUT "Total volume: $volume_total";
$cog_volume_total = scale_3d ($cog_volume_total, 1/$volume_total);
say STDOUT 'CoG of Total volume: ' . join (', ', @{$cog_volume_total});

0;
