#!/usr/bin/perl

eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use warnings;

use lib 'lib';
use File::DXF;
use File::DXF::Util qw /read_DXF/;
use File::DXF::Math qw (scale_3d add_3d distance_3d point_is_in_triangle flatten_triangle);
use 5.010;

die "Usage: $0 IN.DXF OUT.DXF"
    unless (scalar @ARGV == 2);

my @points_3d = ();
my @points_2d = ();

my $dxf_in = new File::DXF;
my $dxf_out = new File::DXF;

$dxf_in->Process (read_DXF ($ARGV[0], 'CP1252'));
my $entities = $dxf_in->{ENTITIES};

my @mesh = $entities->Extract_Meshes;
my $mesh = shift @mesh;

my $nodes_3d = [map {[split / /, $_]} @{$mesh->[0]}];
my $triangles_3d = [map {[split / /, $_]} @{$mesh->[1]}];

# FIXME shouldn't have to convert quads into triangles
my $temp;
for my $t (@{$triangles_3d})
{
    push @{$temp}, $t if scalar @{$t} == 3;
    push @{$temp}, [$t->[0], $t->[1], $t->[2]], [$t->[2], $t->[3], $t->[0]] if scalar @{$t} ==4;
}
$triangles_3d = $temp;

for my $item ($entities->Report ('POINT'))
{
    push @points_3d, $item->Coordinates;
}

my $ein_nodes;

for my $node_id (1 .. scalar @{$nodes_3d})
{
    my $node = $nodes_3d->[$node_id -1];
    $ein_nodes->{$node_id} = [@{$node}];
}

# will be used as a 'seen' log
my $seen;
for my $id_node (keys %{$ein_nodes})
{
    $seen->{$id_node} = undef;
}

# draw first edge on y-axis as a seed

# get three nodes

my $nodes = $triangles_3d->[0];

print STDERR join ' ', 'First triangle:', @{$nodes}, "\n" if $ENV{DEBUG};

my $a_3d = $nodes_3d->[$nodes->[0] -1];
my $b_3d = $nodes_3d->[$nodes->[1] -1];
my $c_3d = $nodes_3d->[$nodes->[2] -1];

my $distance_A = distance_3d ($b_3d, $c_3d);
my $distance_B = distance_3d ($c_3d, $a_3d);
my $distance_C = distance_3d ($a_3d, $b_3d);

$ein_nodes->{$nodes->[0]} = [0.0, 0.0, 0.0];
$ein_nodes->{$nodes->[1]} = [0.0, $distance_C, 0.0];

my $node_0 = $ein_nodes->{$nodes->[0]};
my $node_1 = $ein_nodes->{$nodes->[1]};
my $node_2 = $ein_nodes->{$nodes->[2]};

$seen->{$nodes->[0]} = 1;
$seen->{$nodes->[1]} = 1;

# keep going until all nodes have been flattened

my $dirty = 1;
while ($dirty == 1)
{
    $dirty = 0;

    # iterate through all the triangles in the first group

    for my $triangle (0 .. scalar @{$triangles_3d} -1)
    {
        # get three nodes
        my $nodes = $triangles_3d->[$triangle];

        $node_0 = $ein_nodes->{$nodes->[0] +1};
        $node_1 = $ein_nodes->{$nodes->[1] +1};
        $node_2 = $ein_nodes->{$nodes->[2] +1};

        # count how many of these nodes have already been flattened

        my @done_id;
        push @done_id, $nodes->[0] +1 if $seen->{$nodes->[0] +1};
        push @done_id, $nodes->[1] +1 if $seen->{$nodes->[1] +1};
        push @done_id, $nodes->[2] +1 if $seen->{$nodes->[2] +1};

        # we can only flatten this triangle if two nodes already flat

        next if (scalar @done_id) != 2;

        print STDERR join ' ', 'Triangle:', @done_id, "\n" if $ENV{DEBUG};

        # figure out which nodes need flattening

        my ($a_id, $b_id, $c_id);
        if (!$seen->{$nodes->[0] +1})
        {
            $a_id = $nodes->[1] +1;
            $b_id = $nodes->[2] +1;
            $c_id = $nodes->[0] +1;
        }
        elsif (!$seen->{$nodes->[1] +1})
        {
            $a_id = $nodes->[2] +1;
            $b_id = $nodes->[0] +1;
            $c_id = $nodes->[1] +1;
        }
        elsif (!$seen->{$nodes->[2] +1})
        {
            $a_id = $nodes->[0] +1;
            $b_id = $nodes->[1] +1;
            $c_id = $nodes->[2] +1;
        }

        my $a_3d = $nodes_3d->[$a_id -1];
        my $b_3d = $nodes_3d->[$b_id -1];
        my $c_3d = $nodes_3d->[$c_id -1];

        my $a_2d = [@{$ein_nodes->{$a_id}}];
        my $b_2d = [@{$ein_nodes->{$b_id}}];

        my $midpoint_2d = scale_3d (add_3d ($a_2d, $b_2d), 1/2);

        for my $point (@points_3d)
        {
            next unless point_is_in_triangle ($a_3d, $b_3d, $c_3d, $point);
            my $point_2d = flatten_triangle ($a_3d, $b_3d, $point, $a_2d, $b_2d);
            push @points_2d, $point_2d;
        }

        my $c_2d = flatten_triangle ($a_3d, $b_3d, $c_3d, $a_2d, $b_2d);

        my $distance_A_flat = distance_3d ($b_2d, $c_2d);
        my $error = abs (1 - (distance_3d ($b_3d, $c_3d) / $distance_A_flat));

        print STDERR join ' ', 'Error:', sprintf ("%.12f", $error), "($a_id $b_id $c_id)", "\n";

        # update output file

        $ein_nodes->{$c_id} = [@{$c_2d}];
        $seen->{$c_id} = 1;

        # set unfinished flag

        print STDERR "\n" if $ENV{DEBUG};
        $dirty = 1;
    }
}

for my $point_2d (@points_2d)
{
    push @{$dxf_out->{ENTITIES}},
        {type => 'POINT',
        _data => [[8, 0], [10, $point_2d->[0]], [20, $point_2d->[1]], [30, 0.0]]};
}

# mesh header
my $polyline = {type => 'POLYLINE', _data => [
    [8, 'mesh'],
    [62, 0],
    [66, 1],
    [10, 0.0],
    [20, 0.0],
    [30, 0.0],
    [70, 64],
    [71, scalar (keys %{$ein_nodes})],
    [72, scalar @{$triangles_3d}]
]};

push @{$dxf_out->{ENTITIES}}, $polyline;

# mesh points
for my $node_id (sort {$a <=> $b} keys %{$ein_nodes})
{
    my $vertex = {type => 'VERTEX', _data => [
        [8, 'mesh'],
        [10, $ein_nodes->{$node_id}->[0]],
        [20, $ein_nodes->{$node_id}->[1]],
        [30, $ein_nodes->{$node_id}->[2]],
        [70, 192]
    ]};

    push @{$dxf_out->{ENTITIES}}, $vertex;
}

# mesh triangles
for my $triangle_id (0 .. scalar @{$triangles_3d} -1)
{
    my $node_ids = $triangles_3d->[$triangle_id];

    my $vertex = {type => 'VERTEX', _data => [
        [8, 'mesh'],
        [62, 0],
        [10, 0.0],
        [20, 0.0],
        [30, 0.0],
        [70, 128],
        [71, $node_ids->[0] +1],
        [72, $node_ids->[1] +1],
        [73, $node_ids->[2] +1],
    ]};
#    push @{$vertex->{_data}}, [74, $lookup->{$nodes->[3]}] if defined $nodes->[3];

    push @{$dxf_out->{ENTITIES}}, $vertex;
}

# mesh footer
push @{$dxf_out->{ENTITIES}}, {type => 'SEQEND', _data => []};

open DXF ,">".$ARGV[1];
binmode DXF, ":crlf";
print DXF $dxf_out->DXF;
close DXF;

0;

