package File::DXF::ENTITIES::Entity;

=head1 NAME

File::DXF::ENTITIES::Entity - Generic AutoCAD DXF entity

=head1 SYNOPSIS

This is an unfinished DXF parser

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Exporter;
use vars qw /@ISA @EXPORT_OK/;

@ISA = qw /Exporter/;
@EXPORT_OK = qw /next_item pi distance/;

=pod

Create one like so:

  $foo = new File::DXF::ENTITIES::$bar;

=cut

sub new
{
    my $class = shift;
    $class = ref $class || $class;
    my $self = {};

    bless $self, $class;
    return $self;
}

=pod

Try and find some structure in the raw data

  $foo->Process ($data);

=cut

sub Process
{
    my $self = shift;
    my $data = shift;
}

sub Coordinates
{
    my $self = shift;
    return [$self->{x}, $self->{y}, $self->{z}];
}

sub Normal
{
    my $self = shift;
    return [0, 0, 1] unless defined $self->{normal_x};
    return [$self->{normal_x}, $self->{normal_y}, $self->{normal_z}];
}

sub Type
{
    my $self = shift;
    my $type = ref $self;
    $type =~ s/.*:://;
    return $type;
}

sub distance
{
    my ($a, $b) = @_;
    return sqrt ( (($a->[0] - $b->[0]) * ($a->[0] - $b->[0]))
                + (($a->[1] - $b->[1]) * ($a->[1] - $b->[1]))
                + (($a->[2] - $b->[2]) * ($a->[2] - $b->[2])) );
}

sub pi
{
    return 3.1415926535897932384626433832795;
}

sub next_item
{
    my $data = shift;
    my $item = shift @{$data};
    return @$item;
}

1;

