package File::DXF::ENTITIES::LINE;

=head1 NAME

File::DXF::ENTITIES::LINE - AutoCAD DXF LINE entity

=head1 SYNOPSIS

This is an unfinished DXF parser

=head1 DESCRIPTION

=cut

use strict;
use warnings;

use File::DXF::ENTITIES::Entity ('next_item', 'distance');

use vars qw /@ISA/;
@ISA = qw /File::DXF::ENTITIES::Entity/;

=pod

Create one like so:

  $foo = new File::DXF::ENTITIES::LINE;

=cut

sub new
{
    my $class = shift;
    $class = ref $class || $class;
    my $self = { x => 0,
                 y => 0,
                 z => 0,
                x1 => 0,
                y1 => 0,
                z1 => 0 };

    bless $self, $class;
    return $self;
}

=pod

Try and find some structure in the raw data

  $foo->Process ($data);

=cut

sub Process
{
    my $self = shift;
    my $data = shift;

    while (@{$data})
    {
        my ($code, $string) = next_item ($data);
        $self->{x} = $string if ($code == 10);
        $self->{y} = $string if ($code == 20);
        $self->{z} = $string if ($code == 30);
        $self->{x1} = $string if ($code == 11);
        $self->{y1} = $string if ($code == 21);
        $self->{z1} = $string if ($code == 31);
    }
}

sub Coordinates
{
    my $self = shift;
    my $index = shift or return [$self->{x}, $self->{y}, $self->{z}];
    return [$self->{x}, $self->{y}, $self->{z}] if $index == 0;
    return [$self->{x1}, $self->{y1}, $self->{z1}] if $index == 1;
}

sub Length
{
    my $self = shift;
    return distance ( $self->Coordinates(0), $self->Coordinates(1));
}

1;

